#include <nwnet.h>
#include <stdio.h>

#define FALSE 0
#define TRUE 1

// ******************************
// Example code for Add Partition
// ******************************

typedef struct { char *name;
		 char *value;
		 int  syntax;
	       } ATTRS;

void main()
{ NWDSContextHandle   nContext;
  register int        i;
  int                 err;
  uint8               more = FALSE;
  int32               iterationHandle = NO_MORE_ITERATIONS;
  Buf_T               *AttrBuf;
  size_t              bufSize = DEFAULT_MESSAGE_LEN;
  ATTRS 	      attrList[] =
     {  {"Object Class", "Organizational Unit", SYN_CLASS_NAME},
	{"Description", "Your basic company organization.", SYN_PR_STRING},
	{"Telephone Number",     "429-1212", SYN_TEL_NUMBER}
     };
  uint32              numAttrs = sizeof(attrList)/ sizeof(attrList[0]);

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("Could not create context\n");
      goto EndProc;
    }

  // Set context as desired if necessary and check for errors.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"O=Novell.C=US");
  if(err<0)
    { printf("Error in NWDSSetContext: <%d>\n",err);
      goto EndProc;
    }

  // Allocate space for input buffer and check for errors.
  err = NWDSAllocBuf(bufSize, &AttrBuf);
  if(err < 0)
    { printf("ERROR %d in NWDSAllocBuf\n", err);
      goto EndProc;
    }

  // Initialize input buffer and check for errors.
  err = NWDSInitBuf(nContext, DSV_ADD_PARTITION, AttrBuf);
  if(err < 0)
    { printf("ERROR %d in NWDSInitBuf\n",err);
      goto EndProc;
    }

  // Load the input buffer with the parameters for the new partition
  // checking for errors as you go.  Note the required sequence of
  // attribute name followed by value.
  for(i = 0; i < (int)numAttrs; i++)
    { err = NWDSPutAttrName(nContext, AttrBuf, attrList[i].name);
      if(err < 0)
	{ printf("ERROR %d in NWDSPutAttrName\n", err);
	  goto EndProc;
	}
      err = NWDSPutAttrVal(nContext, AttrBuf, attrList[i].syntax,
							attrList[i].value);
      if(err < 0)
	{ printf("ERROR %d in NWDSPutAttrValue\n", err);
	  goto EndProc;
	}
    }

  // Execute the function to add the partition and check for errors.
  err = NWDSAddPartition(nContext, "CN=GILA.O=Novell", "OU=ZZ.O=Novell",
					&iterationHandle, more, AttrBuf);
  if(err < 0)
    { printf("ERROR %d in NWDSAddPartition\n", err);
      goto EndProc;
    }

  // Perform necessary cleanup.
EndProc:
  NWDSFreeBuf(AttrBuf);
  NWDSFreeContext(nContext);
}
