//   ͻ
//                                                                       
//    module:      connect.c                                             
//    abstract:    This module contains the connect code for the NDS     
//                 version of the BROKER Example NLM.                    
//                                                                       
//    environment: NetWare 4.0 ALPHA                                     
//                 Network C for NLMs SDK v4.0 ALPHA Update              
//                 CLib v4.00 ALPHA                                      
//                                                                       
//     This software is provided as is and carries no warranty           
//     whatsoever.  Novell disclaims and excludes any and all implied    
//     warranties of merchantability, title and fitness for a particular 
//     purpose.  Novell does not warrant that the software will satisfy  
//     your requirements or that the software is without defect or error 
//     or that operation of the software will be uninterrupted.  You are 
//     using the software at your risk.  The software is not a product   
//     of Novell, Inc. or any of subsidiaries.                           
//                                                                       
//   Ķ
//    maintenance history:                                               
//    level    date      pi   description                                
//   Ķ
//     001   06/26/92    md   initial release.                           
//   ͼ

#define NULL 0

#include <nwdsapi.h>
#include "newin400.h"
#include "broker.h"
#include "bglobals.h"
#include <stdio.h>
#include <nwipxspx.h>
#include <nwsemaph.h>
#include <conio.h>
#include <process.h>
#include <time.h>
#include <nwenvrn.h>
#include <advanced.h>
#include <malloc.h>
#include <nwmsg.h>
#include <string.h>

#include <sap.h>
#include <nwdir.h>
#include <nwconn.h>

extern LONG oldScreenID, myScreenID, debugScreenID;
static LONG newClientOutOfMemoryCount = 0;
extern BROKER_CALL_BACK brokerCallBack;

void ListenMain(void *indexP)

{

   IPX_HEADER listenHeaders[12];
   IPX_HEADER *thisHeader;
   CINFOPACKET listenPackets[12];
   CINFOPACKET *thisPacket;
   IPX_ECB listenECBs[12];
   IPX_ECB *thisECB, *tempECB;
   IPX_ECB *head = NULL;
   int ccode, index;
   CINFO *newClient;
#ifdef debug
   LONG oldScreenID;
#endif


	indexP = indexP;	// unused pararmeter

   ccode = RenameThread(GetThreadID(), "Broker Listen");


   /* fill out our ecbs and post for listening */
   for(index = 0; index < 12; index++)
      {
      listenECBs[index].queueHead = &head;
      listenECBs[index].semHandle = initializeSem;
      listenECBs[index].socket = INITIALIZE_SOCKET;
      listenECBs[index].fragCount = 2;
      listenECBs[index].fragList[0].fragAddress = &listenHeaders[index];
      listenECBs[index].fragList[0].fragSize = sizeof(IPX_HEADER);
      listenECBs[index].fragList[1].fragAddress = &listenPackets[index];
      listenECBs[index].fragList[1].fragSize = sizeof(CINFOPACKET);
      ccode = IpxReceive(0, &listenECBs[index]);
      }

#ifdef debug
   oldScreenID = GetCurrentScreen();
   if (oldScreenID != debugScreenID)
      SetCurrentScreen(debugScreenID);
   printf("\nListenMain started");
#endif


      if (unInitClientInfoHead == NULL)
         unInitClientInfoHead = &firstUnInitClient;

   while(!shutdown)
      {
      ccode = WaitOnLocalSemaphore(initializeSem);
      ThreadSwitch();
      if (shutdown)
         break;

      thisECB = IpxGetAndClearQ(&head);
      while (thisECB)
         {
         newClient = (CINFO *)malloc(sizeof(CINFO));  /* this goes to sleep */
         if (newClient == NULL)
            {
            printf("\nError allocating new client structure");
            printf("\nNew client out of memory count: %i", ++newClientOutOfMemoryCount);
            continue;
            }

         thisHeader = (IPX_HEADER *)thisECB->fragList[0].fragAddress;
         thisPacket = (CINFOPACKET *)thisECB->fragList[1].fragAddress;

         if (thisPacket->packetType == 1 && newClient != NULL)
            {

            /* now take the information from the packet the client sent to us */
            /* and copy it into a brand new client structure */

            memcpy(newClient->clientName, thisPacket->userName, 49);
            memcpy(&newClient->address, &thisHeader->sourceNet, 12);
            newClient->spaceInUse = thisPacket->spaceRequired;
            newClient->duration = thisPacket->duration;
            newClient->objectID = thisPacket->info[0];
            newClient->next = NULL;

            /* link this new client structure onto a list of uninitialized */
            /* client structures */
            if (*unInitClientInfoHead == NULL)
               {
               _disable();
               *unInitClientInfoHead = newClient;
               _enable();
               }
            else
               {
               _disable();
               newClient->next = *unInitClientInfoHead;
               *unInitClientInfoHead = newClient;
               _enable();
               }

   #ifdef DEBUG
            if (oldScreenID != debugScreenID)
               SetCurrentScreen(debugScreenID);
            printf("\nReceived query packet from uninitialized client");
            printf("\nPlaced uninitialized client structure on unInit list");
   #endif
            /* wake up the list maintenance process. He will finish */
            /* initializing the new client by setting up an account, etc. */
            /* and then link the new client structure to the list of active */
            /* clients. */

            if (!shutdown)
               ccode = SignalLocalSemaphore(clientListSem);
            }


         /* if we didn't have enough memory to allocate the new client */
         /* we ignore this packet and post the ECB for listening. Maybe */
         /* we will have enough memory next time around. */

         tempECB = thisECB;
         thisECB = thisECB->next;
         ccode = IpxReceive(0, tempECB);
         }
      }

   ExitThread(EXIT_THREAD, 0);
}

void ListMain()

{
   CINFO *thisClient, *temp;
   int ccode;
   IPX_ECB sendECB;
   IPX_HEADER sendHeader;
   CINFOPACKET sendPacket;
   unsigned long transportTime;
   int alreadyInitialized;
   unsigned short initSocket = INITIALIZE_SOCKET;
#ifdef DEBUG
   LONG oldScreenID;
#endif

   ccode = RenameThread(GetThreadID(), "Broker List");

   ccode = IpxOpenSocket(&initSocket);

#ifdef DEBUG
   oldScreenID = GetCurrentScreen();
   
   if (oldScreenID != debugScreenID)
      SetCurrentScreen(debugScreenID);
   printf("\nOpenSocket: %#x", ccode);
#endif


   /* initialize the packet header and ECB as much as possible before */
   /* entering the loop. */

   if (!shutdown)
      {
      sendHeader.packetType = 4;
      sendHeader.destSocket = INITIALIZE_SOCKET;
      sendECB.socket = INITIALIZE_SOCKET;
      sendECB.queueHead = NULL;
      sendECB.semHandle = NULL;
      sendECB.fragCount = 2;
      sendECB.fragList[0].fragAddress = &sendHeader;
      sendECB.fragList[0].fragSize = sizeof(IPX_HEADER);
      sendECB.fragList[1].fragAddress = &sendPacket;
      sendECB.fragList[1].fragSize = sizeof(CINFOPACKET);


      if (initClientInfoHead == NULL)
         initClientInfoHead = &firstInitClient;
      }
#ifdef DEBUG
   oldScreenID = GetCurrentScreen();
   
   if (oldScreenID != debugScreenID)
      SetCurrentScreen(debugScreenID);
   printf("\nListMain started");
#endif


   while(!shutdown)
      {
   
      /* go to sleep until there is something for us to do. */

      ccode = WaitOnLocalSemaphore(clientListSem);
		ThreadSwitch();
      while(*unInitClientInfoHead == NULL)
         ThreadSwitchWithDelay();
      if (shutdown)
         break;


#ifdef DEBUG
      oldScreenID = GetCurrentScreen();
   
      if (oldScreenID != debugScreenID)
         SetCurrentScreen(debugScreenID);
      printf("\nListMain awakened");
#endif 

      /* It's possible that uninitialized client structures */
      /* could pile up on us while we're asleep or while we're */
      /* initializing the client during the body of our code. */
      /* If this happens, the WaitOnLocalSemaphore call will */
      /* not cause us to go to sleep until we've cleared all */
      /* the outstanding uninitialized guys. */

      /* unlink him from the uninitialized list */
      _disable();
      thisClient = *unInitClientInfoHead;

      /* this will zero out the uninit list if it is the last node */

      *unInitClientInfoHead = thisClient->next; 
      _enable();

#ifdef DEBUG
      oldScreenID = GetCurrentScreen();
   
      if (oldScreenID != debugScreenID)
         SetCurrentScreen(debugScreenID);
      printf("\nUninitialized client unlinked from unInit list");
#endif

      /* do a scan to see if this guy is already an initialized guy */
      /* If the guy is already initialized we don't link him into the */
      /* initialized list, but we still send him a reply packet. */
      /* This allows the client to confirm we have initialized him */
      /* by sending another initialize packet if he timed out waiting */
      /* for the first one. */

      /* If he is already initialized we just set the CINFO pointer */
      /* to the client's node on the initialized client list. However, */
      /* we still need to free the node we got from the uninitialized */
      /* list, because that node was allocated by the listen process. */


      alreadyInitialized = 0;

      if (firstInitClient != NULL)
         {
         temp = *initClientInfoHead;
         while(temp != NULL && !shutdown)
            {
            if (!memcmp(&temp->address, &thisClient->address, 12))
               {
               _disable();
               alreadyInitialized = 1;
               free(thisClient);
               thisClient = temp;
               _enable();
               break;
               }
            ThreadSwitch();      /* give other guys a chance to run */
            temp = temp->next;
            }
         }

      /* If the guy wasn't initialized, we need to set him up in */
      /* the bindery and link him into the initialized client list. */

      if (!alreadyInitialized)
         {
         ccode = SetupClientInfo(thisClient);
         if (ccode)
            {
#ifdef DEBUG
            if (oldScreenID != debugScreenID)
               SetCurrentScreen(debugScreenID);
#endif
            printf("\nError returned by SetupClientInfo %#x", ccode);
            }

         thisClient->loginTime = time(NULL);
         thisClient->duration++;
         thisClient->duration *= 60;
         thisClient->duration += thisClient->loginTime;
         thisClient->code.status = 0;
         thisClient->shutdownStatus = 100;

         }
      else
         ThreadSwitch();

      /* now we need to send a confirmation packet back to the client */
      /* initialize all the packet stuff we couldn't initialize earlier */

                  
      memcpy(&sendHeader.destNet, &thisClient->address, 10);
      sendHeader.destSocket = INITIALIZE_SOCKET;
      ccode = IpxGetLocalTarget((unsigned char *)&sendHeader.destNet,
                                 &sendECB, &transportTime);
      if (ccode)
         {
#ifdef DEBUG
         if (oldScreenID != debugScreenID)
            SetCurrentScreen(debugScreenID);
#endif
         printf("\nError obtaining immediate address: %#x", ccode);
         }
      
      sendPacket.packetType = 0x02;
      strcpy(sendPacket.userName, thisClient->clientName);
      sendPacket.spaceRequired = thisClient->spaceInUse;
      sendPacket.duration = thisClient->duration;
      memcpy(&sendPacket.info[0], &thisClient->objectID, 262);
      
#ifdef DEBUG

      if (oldScreenID != debugScreenID)
         SetCurrentScreen(debugScreenID);
#endif

      if (!alreadyInitialized)
         printf("\nListMain initialized new client");
      else
         printf("\nListMain processed previously initialized client");

      /* just send the packet and forget about it. If the client */
      /* doesn't recieve it, he will send another initialize packet. */

      ccode = IpxSend(0, &sendECB);
      if (sendECB.status)
         {
#ifdef DEBUG
         if (oldScreenID != debugScreenID)
            SetCurrentScreen(debugScreenID);
#endif
         printf("\nError sending packet: %x", sendECB.status);
         }
      if (!alreadyInitialized && thisClient->spaceInUse)
         {
         /* link him to the initialized list */

         if (firstInitClient == NULL)
            {
            _disable();
            firstInitClient = thisClient;
            firstInitClient->next = NULL;
            _enable();
            }

         else
            {
            _disable();
            thisClient->next = *initClientInfoHead;
            *initClientInfoHead = thisClient;
            _enable();
            }

#ifdef DEBUG

         if (oldScreenID != debugScreenID)
            SetCurrentScreen(debugScreenID);
#endif
         printf("\nClient is now initialized and linked to the init list");
         }
      if (!thisClient->spaceInUse)
         free(thisClient);
      }

   ExitThread(EXIT_THREAD, 0);
}



/* This function calls an external routine called DestroyClient. */
/* DestroyClient must delete the client's account, remove the */
/* client's directories, and free the client's CINFO node. */
/* Note: we are counting on DestroyClient to go to sleep. */

void CleanUpProcess()

{
   CINFO *thisClient;
   CINFO *previousClient;
   CINFO *temp;
   int ccode;
#ifdef DEBUG
   LONG oldScreenID;


   ccode = RenameThread(GetThreadID(), "Broker CleanUp");

   oldScreenID = GetCurrentScreen();
   
   if (oldScreenID != debugScreenID)
      SetCurrentScreen(debugScreenID);
   printf("\nCleanUpProcess started");
#endif

   while(!shutdown)
      {
      WaitOnLocalSemaphore(destroySem);
      ThreadSwitch();
      if (shutdown)
         break;
      _disable();
      previousClient = thisClient = *initClientInfoHead;
      _enable();
      while(thisClient)
         {
         if (shutdown)
            break;

         _disable();

         temp = thisClient;
         thisClient = thisClient->next;
         if (temp->shutdownStatus == 0xff)
            {
            if (temp == *initClientInfoHead)    /* first node */
               previousClient =  *initClientInfoHead = thisClient;
            else
               previousClient->next = temp->next;
            _enable();
#ifdef DEBUG
            if (oldScreenID != debugScreenID)
               SetCurrentScreen(debugScreenID);
            printf("\nCleanUpProcess: destroying client");
#endif
            ccode = DestroyClient(temp);

            }
         else
            {
            previousClient = temp;
            _enable();
            }
         }

      if (shutdown)
         break;
		ThreadSwitch();
      }
   ExitThread(EXIT_THREAD, 0);
}



void QueryMain()

{
   CINFO *thisClient;
   IPX_HEADER listenHeaders[3];
   IPX_HEADER *thisListenHeader;
   IPX_HEADER sendHeader;
   SINFOPACKET listenPackets[3];
   SINFOPACKET *thisQueryPacket;
   SINFOPACKET querySendPacket;
   REQUEST_RESPOND *thisListenPacket;
   QUERY sendSpacePacket;
   SHUTDOWN sendTimePacket;
   IPX_ECB listenECBs[3];
   IPX_ECB *thisListenECB, *tempECB;
   IPX_ECB *head = NULL;
   IPX_ECB sendECB;
   int ccode, index, packetToSend = 0;
   unsigned long transportTime;
   unsigned long availSpace;
#ifdef DEBUG
   LONG oldScreenID;
#endif


   ccode = RenameThread(GetThreadID(), "Broker Query");


   for(index = 0; index < 3; index++)
      {
      listenECBs[index].queueHead = &head;
      listenECBs[index].semHandle = querySem;
      listenECBs[index].socket = QUERY_SOCKET;
      listenECBs[index].fragCount = 2;
      listenECBs[index].fragList[0].fragAddress = &listenHeaders[index];
      listenECBs[index].fragList[0].fragSize = sizeof(IPX_HEADER);
      listenECBs[index].fragList[1].fragAddress = &listenPackets[index];
      listenECBs[index].fragList[1].fragSize = sizeof(SINFOPACKET);
      ccode = IpxReceive(0, &listenECBs[index]);
      }

   sendECB.queueHead = NULL;
   sendECB.socket = QUERY_SOCKET;
   sendHeader.destSocket = QUERY_SOCKET;
   sendECB.fragCount = 2;
   sendECB.fragList[0].fragAddress = &sendHeader;
   sendECB.fragList[0].fragSize = sizeof(IPX_HEADER);
   sendHeader.packetType = 4;

#ifdef DEBUG
   oldScreenID = GetCurrentScreen();
   
   if (oldScreenID != debugScreenID)
      SetCurrentScreen(debugScreenID);
   printf("\nQueryMain started");
#endif



   while(!shutdown)
      {
      ccode = WaitOnLocalSemaphore(querySem);
      ThreadSwitch();
      if (shutdown)
         break;
      
#ifdef DEBUG
      oldScreenID = GetCurrentScreen();
   
      if (oldScreenID != debugScreenID)
         SetCurrentScreen(debugScreenID);
      printf("\nQueryMain awakened");
#endif

      thisListenECB = IpxGetAndClearQ(&head);
      while(thisListenECB)
         {
      
         thisListenHeader = (IPX_HEADER *)thisListenECB->fragList[0].fragAddress;
         thisListenPacket = (REQUEST_RESPOND *)thisListenECB->fragList[1].fragAddress;

         if (thisListenPacket->packetType == 3 || thisListenPacket->packetType == 5)
            {
            /* now find the client */
            thisClient = *initClientInfoHead;
            while(thisClient != NULL)
               {
               if(!memcmp(&thisClient->address, &thisListenHeader->sourceNet, 10))
                  break;
               _disable();
               thisClient = thisClient->next;
               _enable();
               ThreadSwitch();
               }

            ccode = IpxGetLocalTarget((unsigned char *)&thisClient->address,
                        &sendECB, &transportTime);
            memcpy(&sendHeader.destNet, &thisListenHeader->sourceNet, 10);

            }


         if (thisClient != NULL || thisListenPacket->packetType == 1)
            {

            /* check for the initial client query */

            if (thisListenPacket->packetType == 1)
               {
               thisQueryPacket = (SINFOPACKET *)thisListenECB->fragList[1].fragAddress;
               strcpy(querySendPacket.serverName, serverName);


 		         ccode = GetAvailableUserDiskSpace("sys:", &availSpace);

               querySendPacket.spaceAvailable = ((availSpace) / 0x100);

               ccode = IpxGetLocalTarget(
                           (unsigned char *)&thisListenHeader->sourceNet,
                           &sendECB,
                           &transportTime);
               memcpy(&sendHeader.destNet, &thisListenHeader->sourceNet, 10);

               sendECB.fragList[1].fragAddress = &querySendPacket;
               sendECB.fragList[1].fragSize = sizeof(SINFOPACKET);
               querySendPacket.packetType = 2;
               packetToSend = 1;


#ifdef DEBUG
               oldScreenID = GetCurrentScreen();
   
               if (oldScreenID != debugScreenID)
                  SetCurrentScreen(debugScreenID);
               printf("\nQueryMain received init packet");
#endif

               }

            /* check for a time remaining request */

            if (thisListenPacket->packetType == 3)
               {
               sendECB.fragList[1].fragAddress = &sendTimePacket;
               sendECB.fragList[1].fragSize = sizeof(SHUTDOWN);
               sendTimePacket.packetType = 4;
               sendTimePacket.timeOut = (thisClient->duration / ONE_MINUTE);
               packetToSend = 1;

#ifdef DEBUG
               oldScreenID = GetCurrentScreen();
   
               if (oldScreenID != debugScreenID)
                  SetCurrentScreen(debugScreenID);
               printf("\nQueryMain received time remaining request");
#endif

               }

            /* check for a space remaining request */

            if (thisListenPacket->packetType == 5)
               {
               sendECB.fragList[1].fragAddress = &sendSpacePacket;
               sendECB.fragList[1].fragSize = sizeof(QUERY);
               sendSpacePacket.packetType = 6;
               memcpy(sendSpacePacket.info, &thisClient->volumeNumber, 16);
               packetToSend = 1;
#ifdef DEBUG
               oldScreenID = GetCurrentScreen();
   
               if (oldScreenID != debugScreenID)
                  SetCurrentScreen(debugScreenID);
               printf("\nQueryMain received space remaining request");
#endif

               }

            if (packetToSend)
               ccode = IpxSend(0, &sendECB);
            }

         tempECB = thisListenECB;
         thisListenECB = thisListenECB->next;
         ccode = IpxReceive(0, tempECB);
         if (shutdown)
            break;
         }
      if (shutdown)
         break;
		ThreadSwitch();
      }

   ExitThread(EXIT_THREAD, 0);
}




void ClientUpDateProcess(BROKER_CALL_BACK *callBack)

{
   CINFO *thisClient;
   int ccode, remainingMinutes;
   BYTE resultList[5];
   char broadcastString[58];
   LONG directories, files, blocks;
   WORD connectionList[5], numberOfConnections;
   time_t currentTime;

   while(!shutdown)
      {
      if (callBack->clientHead != NULL)
         {
         thisClient = *callBack->clientHead;
         while(thisClient != NULL )
            {

            if (shutdown)
               break;
            
            remainingMinutes = 0;
            currentTime = time(NULL);
            if (thisClient->duration - currentTime <= 600)
               {

               ccode = GetObjectConnectionNumbers(thisClient->clientName,
                                                  1,
                                                  &numberOfConnections,
                                                  connectionList,
                                                  5);

               remainingMinutes = 10;
               if ((thisClient->duration - currentTime) <= 420)
                  remainingMinutes = 7;
               if ((thisClient->duration - currentTime) <= 300)
                  remainingMinutes = 5;
               if ((thisClient->duration - currentTime) <= 120)
                  remainingMinutes = 2;
               if ((thisClient->duration - currentTime) <= 60)
                  {
                  thisClient->shutdownStatus = 0xff;
                  _disable();
                  thisClient = thisClient->next;
                  _enable();

                  SignalLocalSemaphore(destroySem);
                  goto ReSchedule;
                  }
               else
                  {
                  sprintf(broadcastString,
                        "BROKER will destroy you in less than %i minutes",
                        remainingMinutes);

                  ccode = SendBroadcastMessage(broadcastString,
                                             connectionList,
                                             resultList,
                                             5);
                  }   


               }

            if (thisClient->shutdownStatus != 0xff && thisClient->shutdownStatus != -2)
               if (!shutdown)
                  {
                  ccode = GetDiskUtilization(thisClient->objectID,
                                             thisClient->volumeNumber,
                                             &directories,
                                             &files,
                                             &blocks);
                  }

            if (thisClient->shutdownStatus != 0xff && thisClient->shutdownStatus != -2)
               {
               thisClient->directoriesInUse = directories;
               thisClient->filesInUse = files;
               thisClient->blocksInUse = blocks;

               }


            _disable();
            thisClient = thisClient->next;
            _enable();

            if (shutdown)
               break;

            }
          if (shutdown)
            break;
         }
ReSchedule:
      if (!shutdown)
         {
         ScheduleSleepAESProcessEvent((struct AESProcessStructure *)callBack);
   	   return;
         }
      }
   return;
}





void ConnectMain()

{
   int ccode;
   unsigned short initSocket = INITIALIZE_SOCKET;
   unsigned short querySocket = QUERY_SOCKET;
   LONG connectRTag;

   oldScreenID = myScreenID = GetCurrentScreen();
#ifdef DEBUG
   debugScreenID = CreateScreen("Broker Debug Screen", 0);
#endif



   ccode = RenameThread(GetThreadID(), "Broker Connect");


   if (oldScreenID != myScreenID)
      SetCurrentScreen(myScreenID);
   printf("\nBroker Connection Manager Initializing ...");


   /* since we are just starting up, we know we don't have any new */
   /* uninitialized clients. So we set the head of the uninitialized */
   /* list to null. */
   
   firstUnInitClient = NULL;
   unInitClientInfoHead = &firstUnInitClient;


   /* we do the same for the initialized client list */

   firstInitClient = NULL;
   initClientInfoHead = &firstInitClient;

   /* allocate a local semaphore for the initialize socket */
   /* we set the initial value to zero so only one process */
   /* may gain access to the semaphore at a time. The CLIB */
   /* IPX module signals the semaphore when the ECB is ready */
   /* for us. */

   /* start the client list maintenance thread */

   clientListSem = OpenLocalSemaphore(0L);
   ccode = BeginThread(ListMain, NULL, 0x5000, NULL);

   /* start the query manager thread */

   ccode = IpxOpenSocket(&querySocket);
   querySem = OpenLocalSemaphore(0L);
   ccode = BeginThread(QueryMain, NULL, 0x4000, NULL);

   /* now that the client list and maintenance and query threads are started, */
   /* we can start a thread to listen for client initialization */
   /* packets. */

   ccode = IpxOpenSocket(&initSocket);
   initializeSem = OpenLocalSemaphore(0L);
   ccode = BeginThread(ListenMain, NULL, 0x4000, NULL);


   /* start the clean up process. This guy will walk the list */
   /* of initialized clients searching for guys with a shutdown */
   /* status of -1. If he finds any of these guys, he will unlink */
   /* them and destroy their accounts and directories/ */

   destroySem = OpenLocalSemaphore(0L);
   ccode = BeginThread(CleanUpProcess, NULL, 0x4000, NULL);


   /* now we start an asynchronous process to update certain client */
   /* statistics. */

   connectRTag = AllocateResourceTag(GetNLMHandle(),
                                     "Client Update Process",
                                      AESProcessSignature);
   brokerCallBack.AWakeUpDelayAmount = 0x438;  /* every 60 seconds */
   brokerCallBack.AProcessToCall = ClientUpDateProcess;
   brokerCallBack.ARTag = connectRTag;
   brokerCallBack.AOldLink = NULL;
   brokerCallBack.clientHead = initClientInfoHead;

   ScheduleSleepAESProcessEvent((struct AESProcessStructure *)&brokerCallBack);


/* put code here to display initialized client structures */
   while(!shutdown)
      {
      delay(1);
      ThreadSwitch();
      }
   return;

}
