#include <nwnet.h>
#include <stdio.h>
#include <mem.h>

// *****************************
// Example code for Define Class
// *****************************
void main()
{ NWDSContextHandle nContext;
  int               err;
  Class_Info_T      classInfo;
  Buf_T             *ClassBuf;

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("ERROR Could not create context\n");
      goto EndProc;
    }

  // Set context and check for errors.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"O=Novell.C=US");
  if(err<0)
    { printf("Error in NWDSSetContext: <%d>\n",err);
      goto EndProc;
    }

  // Allocate space for result buffer and check for errors.
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &ClassBuf);
  if(err<0)
    { printf("Error in NWDSAllocBuf: <%d>\n",err);
      goto EndProc;
    }

  // Initialize the ClassBuf buffer using the DSA_DEFINE_CLASS operation.
  // Check for errors.
  err = NWDSInitBuf(nContext, DSV_DEFINE_CLASS, ClassBuf);
  if(err<0)
    { printf("Error in NWDSInitBuf: <%d>\n",err);
      goto EndProc;
    }

  // Set the classInfo variables according to the rules defined in the API
  // documentation.
  classInfo.classFlags = 0;
  classInfo.asn1ID.length = 8;
  memset(classInfo.asn1ID.data,0x00,8);

  // Fill the ClassBuf buffer with the information specified in the API
  // documentation.  This task consists of five calls to NWDSBeginClassItem
  // and one call to NWDSPutClassItem for each attribute or class name string.

  // Entries for Super Class Names.  Start section by calling to
  // NWDSBeginClassItem.  Check for errors.
  err = NWDSBeginClassItem(nContext,ClassBuf);
  if(err<0)
    { printf("Error in NWDSBeginClassItem: <%d>\n",err);
      goto EndProc;
    }
  // Put "Person" in as the value of the super class.  Check for errors.
  err = NWDSPutClassItem(nContext,ClassBuf,"Person");
  if(err<0)
    { printf("Error in NWDSPutClassItem: <%d>\n",err);
      goto EndProc;
    }

  // Entries for Containment Class Names.  Start section by calling to
  // NWDSBeginClassItem.  Check for errors.
  err = NWDSBeginClassItem(nContext,ClassBuf);
  if(err<0)
    { printf("Error in NWDSBeginClassItem: <%d>\n",err);
      goto EndProc;
    }

  // Entries for Naming Attribute Names.   Start section by calling to
  // NWDSBeginClassItem.  Check for errors.
  err = NWDSBeginClassItem(nContext,ClassBuf);
  if(err<0)
    { printf("Error in NWDSBeginClassItem: <%d>\n",err);
      goto EndProc;
    }

  // Entries for Mandatory Attribute Names.  Start section by calling to
  // NWDSBeginClassItem.  Check for errors.
  err = NWDSBeginClassItem(nContext,ClassBuf);
  if(err<0)
    { printf("Error in NWDSBeginClassItem: <%d>\n",err);
      goto EndProc;
    }

  // Entries for Optional Attribute Names.  Start section by calling to
  // NWDSBeginClassItem.  Check for errors.
  err = NWDSBeginClassItem(nContext,ClassBuf);
  if(err<0)
    { printf("Error in NWDSPutClassItemCount: <%d>\n",err);
      goto EndProc;
    }

  // Call to NWDSDefineClass.  Check for errors.
  err = NWDSDefineClass(nContext, "ANewClass", &classInfo, ClassBuf);
  if(err<0)
    { printf("Error in NWDSDefineClass: <%d>\n",err);
      goto EndProc;
    }

  // Clean up buffer and context.  These functions do not return anything.
EndProc:
  NWDSFreeContext(nContext);
  NWDSFreeBuf(ClassBuf);
}
