
/**/
/*                                                                       */
/*                IPX Based Time Server                                  */
/*                                                                       */
/*   This routine will act as a time server.  It will use a well         */
/*   know socket to service requests.  It will accept IPX datagram       */
/*   time server requests on this socket.  It will reply to these        */
/*   requests with the current time returned in the IPX data portion.    */
/*   It will then resume listening for more requests.                    */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <time.h>
#include <fcntl.h>
#include <tiuser.h>
#include <tispxipx.h>
#include <sap.h>
#include "mrdnxt.h"   /* Missing nxt.h declarations       */
#include "timeserv.h" /* Timeserver specific declarations */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);

main()
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* TZ environment variable                    */
	IPXAddress  myaddress;    /* My network,node,socket                     */
	WORD        serverSocket; /* Temp holding area for server socket        */

/*   ECB and Data buffers for listening and sending                        */

	ECB         ipx_ECB;		         /* Event Control Block                 */
	IPXHeader   ipx_header;          /* 1st data buf, IPX header            */
	BYTE        ipx_data[512] = {0}; /* 2cd data buf, User data             */

	int         status;       /* scratch status                             */
	int         i;            /* scratch index                              */

/*  */

/*   Get and display the time   */

	time_zone = getenv("TZ");
	if (time_zone == NULL)
		{
		printf("\nTime zone variable (TZ=) must be set before running server\n");
		printf("Time server aborted\n\n");
		return;
		}
	else
		printf("\nTime zone is %s\n",time_zone);

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is %s\n", asctime(time_block) );

/*   Initialize the IPX function library   */

	if ((status=IPXInitialize()) != SUCCESSFUL )
		{
		printf("IPXInitialize fails, status = 0x%x, program aborted\n\n",status);
		return;
		}

/*
 *   Open a well known socket for the timeserver to use.
 *   We open this socket as SHORT_LIVED, it will close on program termination.
 */
	serverSocket=TIME_SERVER_SOCKET;
	memcpy(myaddress.socket,&serverSocket,2);
	if ((status=IPXOpenSocket(myaddress.socket, SHORT_LIVED )) != SUCCESSFUL)
		{
		printf("IPXOpenSocket fails, status = 0x%x, program aborted\n\n",status);
		return;
		}
/*
 *   Get and display the local network and node and socket.
 */
	IPXGetInternetworkAddress( (BYTE *)&myaddress);

	printf("IPX based timeserver started on :\n\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);
	printf("\n\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);
	printf("\n\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n\nPress any key to abort program\n\n");

/*
 *   Prepare an ECB.  This will be used for listening and sending.
 *   There will be two fragments, one for the IPX header, and one
 *   for the data to send or recieve.
 */
	ipx_ECB.ESRAddress    = (void *) NULL;    /* do not call ESR on completion */
	ipx_ECB.fragmentCount = 2;	
	ipx_ECB.fragmentDescriptor[0].address = &ipx_header;
	ipx_ECB.fragmentDescriptor[0].size    = sizeof(IPXHeader);
	ipx_ECB.fragmentDescriptor[1].address = ipx_data;
	ipx_ECB.fragmentDescriptor[1].size    = 512;

/*
 *    This is the main server loop.
 *    Loop until a key is pressed, then leave.
 */
	while (!kbhit())
		{
		memcpy(&ipx_ECB.socketNumber,myaddress.socket,2); /* listen on my socket */
		IPXListenForPacket(&ipx_ECB);                     /* Post the listen ECB */

	/*
	 *   Loop, When something comes in it will reset the inUseFlag.
	 *
	 *   The provider will use one (not necessarily the last) ECB
	 *   posted to fulfill the request, but since we post only one
	 *   we know this is the one chosen.
	 */
		while (ipx_ECB.inUseFlag != AVAILABLE_FOR_USE )
			{
			IPXRelinquishControl();  /* Give IPX (and others) the CPU       */
			if (kbhit())             /* Cancel the listen, exit the program */
				{
				status=IPXCancelEvent(&ipx_ECB);
				printf("Program canceled from keyboard\n");
				printf("IPX Listen cancelled, IPX Cancel status =0x%02x\n\n",status);
				return;
				}
			}

		if (ipx_ECB.completionCode != SUCCESSFUL)
			{
			printf("IPXListenForPacket failed, program aborted\n");
			printf(" completion code = 0x%02x\n\n",ipx_ECB.completionCode);
			return;
			}

	/*   Check to see if this is really a time request   */

		printf("Packet receive on time server socket from :\n\tNetwork number = 0x");
		for (i=0;i<4;i++)printf("%02x",ipx_header.source.network[i]);
		printf("\n\tNode number    = 0x"); 
		for (i=0;i<6;i++)printf("%02x",ipx_header.source.node[i]);
		printf("\nPacket :\n");
		DumpIPXpacket( &ipx_header, ipx_data, 16);

		if (ipx_data[0] != TIME_REQUEST)
			printf("\nNon TIME REQUEST received on socket, ignored\n\n");
		else
			{
		/*
		 *   This seems to be a valid request, build a time packet and
		 *   send it back to the client.
		 *
		 *   We can use the same ECB, in fact the immediateAddress
		 *   field we will leave alone. This way the packet is sent back
		 *   the same way it came, either directly to the client or through
		 *   some local bridge.
		 *
		 *   We will also use the same IPX header buffer but first we must
		 *   move the source to the destination field of the IPX header,
		 *   send it back from whence it came.  Also, fill in my address
		 *   as the source.
		 */
			printf("\nValid TIME REQUEST received, processing now.\n");

			ipx_header.packetType = IPX_PACKET_TYPE;
			memcpy( &ipx_header.destination, &ipx_header.source, sizeof(IPXAddress)); 
			memcpy( &ipx_header.source,      &myaddress,         sizeof(IPXAddress));

			memcpy(&ipx_ECB.socketNumber,myaddress.socket,2); /* source socket */
			/* ipx_ECB.immediateAddress = leftover from request packet         */
		
		/*
		 *   Get the time into the data area at the last possible moment
		 *   and then send the time packet on its merry way.
		 */
			ipx_data[0] = TIME_REPLY;              /* time reply operation   */
			time_of_day = time(NULL);              /* get latest time        */
			time_zone = getenv("TZ");				   /* and time zone          */
			memcpy(&ipx_data[1], &time_of_day, sizeof(time_t));
			strcpy(&ipx_data[1+sizeof(time_t)],time_zone);
			IPXSendPacket(&ipx_ECB);

		/*
		 *  Check to see if the packet was sent
		 *  Successful means sent, but not necessarily recieved
		 */
			while(ipx_ECB.inUseFlag != AVAILABLE_FOR_USE )
				{
				IPXRelinquishControl();  /* Give IPX (and others) the CPU     */
				if (kbhit())             /* Cancel the send, exit the program */
					{
					status=IPXCancelEvent(&ipx_ECB);
					printf("\nProgram canceled from keyboard\n");
					printf("IPX Send cancelled, IPX Cancel status =0x%02x\n\n",status);
					return;
					}
				}

			if (ipx_ECB.completionCode != SUCCESSFUL)
				{
				printf("\nIPXSendPacket failed, program aborted\n",ipx_ECB.completionCode);
				printf(" completion code = 0x%02x\n\n",ipx_ECB.completionCode);
				return;
				}

			printf("TIME REPLY sent successfully.\n\n");
			printf("Press any key to abort program\n\n");

			} /* valid TIME REQUEST */
			
		} /* end while !kbhit() */

	printf("\nProgram canceled from keyboard\n\n");

return;			
}  /* end main() */

