// ******************************************
// Sample Code for Modify Object
// The attribute that will be modified is ACL
// ******************************************
#include <nwnet.h>
#include <caherr.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <malloc.h>

#define FALSE 0
#define TRUE 1

// **********************************************
// Main Function
// **********************************************
void main()
{ int 			err=0;
  NWDSContextHandle 	nContext;
  Buf_T 		*ChngBuf=NULL;
  Object_ACL_T 		ACLRec;

  err = NWInitUnicodeTables(001,437);
  if(err)
    { printf("ERROR %d NWInitUnicodeTables\n",err);
      goto EndProc;
    }

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("Could not create context\n");
      exit(0);
    }

  // Set context as desired if necessary and check for errors.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"O=Novell");
  if(err<0)
    { printf("ERROR: %d SetContext\n",err);
      goto EndProc;
    }

  // Allocate space for ChngBuf and check for errors.
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &ChngBuf);
  if(err<0)
    { printf("ERROR: %d AllocBuf\n",err);
      goto EndProc;
    }

  // Initialize ChngBuf and check for errors.  This buffer will be
  // initialized to the DSV_MODIFY_ENTRY operation.
  err = NWDSInitBuf(nContext, DSV_MODIFY_ENTRY, ChngBuf);
  if(err<0)
    { printf("ERROR: %d InitBuf\n", err);
      goto EndProc;
    }

  // Place ACL modifications into ChngBuf.  This is done by first calling
  // NWDSPutChange with the name of the attribute to be changed, followed
  // followed by a call to NWDSPutAttrVal for each value to be added.
  // This example adds additional ACL values to the object CN=Guest.O=Novell.
  // Two values are added:
  //    giving [Public] compare and read rights to the Surname attribute
  //    giving O=Novell browse, add, rename rights to the object itself.
  // Check for errors.
  err = NWDSPutChange(nContext,ChngBuf,DS_ADD_VALUE,"ACL");
  if(err<0)
    { printf("ERROR: %d PutChange\n", err);
      goto EndProc;
    }

  ACLRec.protectedAttrName = "Surname";
  ACLRec.subjectName = "[Public]";
  ACLRec.privileges = DS_ATTR_COMPARE|DS_ATTR_READ;
  err = NWDSPutAttrVal(nContext,ChngBuf,SYN_OBJECT_ACL,(void *)&ACLRec);
  if(err<0)
    { printf("ERROR: %d PutAttrVal\n", err);
      goto EndProc;
    }

  ACLRec.protectedAttrName = "[Entry Rights]";
  ACLRec.subjectName = ".O=Novell";
  ACLRec.privileges = DS_ENTRY_BROWSE|DS_ENTRY_ADD|DS_ENTRY_RENAME;
  err = NWDSPutAttrVal(nContext,ChngBuf,SYN_OBJECT_ACL,(void *)&ACLRec);
  if(err<0)
    { printf("ERROR: %d PutAttrVal\n", err);
      goto EndProc;
    }

  // Make the call to NWDSModifyObject.  Check for errors.
  err = NWDSModifyObject(nContext,".CN=Guest.O=Novell", NULL, 0, ChngBuf);
  if(err<0)
    { printf("ERROR: %d ModifyObject\n", err); }

  // Clean up buffers and context.  These functions do not return anything.
EndProc:
  NWDSFreeContext(nContext);
  NWDSFreeBuf(ChngBuf);
  NWFreeUnicodeTables();
}
