#include <nwnet.h>
#include <stdio.h>

#define FALSE 0
#define TRUE  1

// ***************************
// Sample code for Read Object
// ***************************
void main()
{ int               err,i;
  NWDSContextHandle nContext;
  int32             iterHandle=-1L;
  Buf_T             *ResBuf, *NameBuf;
  char              attrName[MAX_DN_CHARS+1];
  uint32            totAttr,ValCount,SynId;
  char              *attributes[] = {"Surname","Telephone Number"};

  // Create context and check for errors.
  nContext = NWDSCreateContext();
  if(nContext==ERR_CONTEXT_CREATION)
    { printf("Could not create context\n");
      goto EndProc;
    }

  // Set context and check for errors. Not necessary if default context is ok.
  err = NWDSSetContext(nContext, DCK_NAME_CONTEXT,"O=Novell.C=US");
  if(err<0)
    { printf("Error in NWDSSetContext: <%d>\n",err);
      goto EndProc;
    }

  // Allocate space for input and output buffers and check for errors.
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &ResBuf);
  if(err<0)
    { printf("Error in NWDSAllocBuf: <%d>\n",err);
      goto EndProc;
    }
  err = NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &NameBuf);
  if(err<0)
    { printf("Error in NWDSAllocBuf: <%d>\n",err);
      goto EndProc;
    }

  // Initialize input buffer and check for errors.  The output buffer does
  // not need to be initialized.   This buffer will be initialized to the
  // DSV_READ operations.
  err = NWDSInitBuf(nContext, DSV_READ, NameBuf);
  if(err<0)
    { printf("Error in NWDSInitBuf: <%d>\n",err);
      goto EndProc;
    }

  // Put object attributes that are to be read into input buffer.  Check
  // for errors.
  for(i=0; i<2; i++)
    { err = NWDSPutAttrName(nContext, NameBuf, attributes[i]);
      if(err<0)
	{ printf("Error in NWDSPutAttrName: <%d>\n",err);
	  goto EndProc;
	}
    }

  // Call to NWDSRead.  iterHandle must be initialized to -1.  This function
  // should be called until iterHandle is -1.  In the case that the result
  // spans multiple output buffers, iterHandle will be something other than
  // NO_MORE_ITERATIONS.  Do not change the value returned.  Just send it
  // back with the next call to NWDSRead.  Only attribute names are
  // requested, so infoType is set to DS_ATTRIBUTE_VALUES.  Since only
  // particular attributes are wanted, allAttrs is set to FALSE.  If all
  // attributes are wanted, set allAttrs to TRUE and don't bother setting
  // up NameBuf.  Just send a NULL for the attrNames buffer parameter in
  // the call the NWDSRead.
  iterHandle = NO_MORE_ITERATIONS;
  do
    { err = NWDSRead(nContext,"CN=SomeObject", DS_ATTRIBUTE_NAMES,
	       FALSE,NameBuf,&iterHandle,ResBuf);
      if(err<0)
	{ printf("Error in NWDSRead: <%d>\n",err);
	  goto EndProc;
	}

      // Pull stuff out of the output buffer, checking for errors.
      // First, get the total number of attributes in the buffer, checking
      // for errors.
      err = NWDSGetAttrCount(nContext,ResBuf,&totAttr);
      if(err<0)
	{ printf("Error in NWDSGetAttrCount: <%d>\n",err);
	  goto EndProc;
	}

      // Next, pull out the attributes in the buffer, checking for errors.
      for(i=0;i<totAttr;i++)
	{ err = NWDSGetAttrName(nContext,ResBuf,attrName,&ValCount,
		    &SynId);
	  if(err<0)
	    { printf("Error in NWDSGetAttrName: <%d>\n",err);
	      goto EndProc;

	    }
	  // Print out the attribute names.
	  printf("Attribute Found : <%s>\n",attrName);
	}
    } while(iterHandle != NO_MORE_ITERATIONS);

  // Clean up buffers and context.  These functions do not return anything.
EndProc:
  NWDSFreeBuf(NameBuf);
  NWDSFreeBuf(ResBuf);
  NWDSFreeContext(nContext);
}
