//   ͻ
//                                                                       
//    module:      setupnew.c                                            
//    abstract:    This module contains the setup for the NDS version    
//                 of the BROKER Example NLM.                            
//                                                                       
//    environment: NetWare 4.0 ALPHA                                     
//                 Network C for NLMs SDK v4.0 ALPHA Update              
//                 CLib v4.00 ALPHA                                      
//                                                                       
//     This software is provided as is and carries no warranty           
//     whatsoever.  Novell disclaims and excludes any and all implied    
//     warranties of merchantability, title and fitness for a particular 
//     purpose.  Novell does not warrant that the software will satisfy  
//     your requirements or that the software is without defect or error 
//     or that operation of the software will be uninterrupted.  You are 
//     using the software at your risk.  The software is not a product   
//     of Novell, Inc. or any of subsidiaries.                           
//                                                                       
//   Ķ
//    maintenance history:                                               
//    level    date      pi   description                                
//   Ķ
//     001   06/26/92    md   initial release.                           
//   ͼ

#define NULL 0

#include "newin400.h"
#include "broker.h"
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>
#include <nwbindry.h>
#include <nwlocale.h>
#include <nwdir.h> 
#include <time.h>
#include <nwipxspx.h>
#include <nwconn.h>
#include <dfs.h>
#include <conio.h>
#include <io.h>
#include <fcntl.h>
#include <string.h>


#define BLOCK_SIZE	 4096
#define SECTOR_SIZE   512
#define REQUEST_TOO_LARGE	11

static int clientNameCounter = 0;	/* int counter; increment for every new client */
extern NWDSContextHandle context;


int DestroyClient(CINFO *oldClient)
{
	int ccode;
   char temp_str1[48];
   char temp_str2[48];


   if (usingNDS)
	{
	ccode = NWDS.RemoveObject(context, oldClient->clientName);
   if (ccode < 0)
      printf ("\nError destroying client object: %i", ccode);
	}
	else
	{
		if ((ccode = DeleteBinderyObject(oldClient->clientName,0x01)) != 0)
			printf("\nError destroying client object");
	}


	 /* Delete client files and client directory; */

   sprintf(temp_str1,"%s%s",oldClient->dirPath,"\\*.*");
	remove(temp_str1);
	rmdir(oldClient->dirPath);

   /* Delete login file and login directory */
   /* this code doesn't make sense for users created with */
   /* Directory APIs. However, we execute it anyway because */
   /* we don't know how this guy was created. If it fails, */
   /* we still continue - it doesn't really matter. */

   sprintf(temp_str2,"%s%s",oldClient->loginPath,"\\*.*");
   remove(temp_str2);
	rmdir(oldClient->loginPath);


  /* Return CINFO pointer; free system memory resource */

  free(oldClient);


	return ccode;

}

/*****************************************************************/
int SetupClientInfo(CINFO *newClient)
{
   int ccode, err = 0, directoryClient = 0; 
   WORD objectType = OT_USER;
   char objectName[48];
   LONG availSpace;
   int volNumber = 0;
   char volName[16] = "sys";
   LONG requestedSpaceIn4KBlocks;
   WORD trusteeRightsMask;
   FILE *fp;
   char map_str[60];
   char temp_str1[60];
   char temp_str2[60];

   int32 iterationHandle;
   char homeDir[60];
   uint8 more;
   Buf_T *userBuffer;
   Stream_T loginStream;
   int handle;

   /* Change the name of the client so it is unique */
   /* define a global variable in bglobals.h and broker.c */

   sprintf(newClient->clientName,"%s%d","client",clientNameCounter++);


   /* if the client is a bindery client, objectID will be zero */

   if (!newClient->objectID)
      {

      /* Create user object in bindery */
      if ((ccode = CreateBinderyObject(newClient->clientName,0x1,
                           BF_STATIC, 0x31)) != 0)

      if (ccode == 238) /* object already exist */
      printf("\nError: object already exits; %d",ccode);
      else
      if (ccode)
      printf("\nError creating bindery object: %d", ccode);


      /* Find the object ID and put it in the client's structure */

      if ((ccode = GetBinderyObjectID(newClient->clientName,0x1,
		  						      (long *)&newClient->objectID)) != 0)
      {
      printf("\nError getting object ID");
		goto CleanUp;

      }

      /* Get bindery object name for (testing) verification */

      ccode = GetBinderyObjectName(newClient->objectID,objectName,&objectType);
      if (ccode != 0)
      printf("\nError getting object name: %3d",ccode);
      }

   /* if the client is a NWDS client, objectID will be 1 */

		else if (usingNDS)
      {

		directoryClient = newClient->objectID;

      err = 1;

      ccode = NWDS.AllocBuf(DEFAULT_MESSAGE_LEN, &userBuffer);
      if (ccode < 0)
         {
         printf("\nError allocating NWDS buffer (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }
      err = 2;

      ccode = NWDS.InitBuf(context, DSV_ADD_ENTRY, userBuffer);
      if (ccode < 0)
         {
         printf("\nError initializing NWDS buffer (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }

      ccode = NWDS.PutAttrName(context, userBuffer, "Object Class");
      if (ccode < 0)
         {
         printf("\nError putting attribute name (object class) \
               in NWDS buffer\n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      ccode = NWDS.PutAttrVal(context, userBuffer, SYN_CLASS_NAME, "User");
      if (ccode < 0)
         {
         printf("\nError putting attribute value (object class) \
               in NWDS buffer\n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      ccode = NWDS.PutAttrName(context, userBuffer, "Surname");
      if (ccode < 0)
         {
         printf("\nError putting attribute name (Surname) \
               in NWDS buffer\n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      ccode = NWDS.PutAttrVal(context, userBuffer, SYN_CI_STRING, "broker");
      if (ccode < 0)
         {
         printf("\nError putting attribute value (surname) \
               in NWDS buffer\n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

		more = 0;
      ccode = NWDS.AddObject(context, newClient->clientName,
                            &iterationHandle, more, userBuffer);

      if (ccode < 0)
         {
         printf("\nError adding user object to Directory \
                  (SetupClientInfo): %i", ccode);
         goto CleanUp;
         }


      /* now we need to get a 'pseudo' bindery object ID for the client */
      /* in order to make file system calls here and elsewhere in the NLM */

      ccode = NWDS.MapNameToID(context, 0,
                              newClient->clientName,
                              &newClient->objectID);
      if (ccode < 0)
         {
         printf("\nError getting user ID (SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      /* We still need to create the user's login script and add that to */
      /* to the userBuffer as an attribute. We will do that later in this */
      /* function. We also need to create the user's home directory.*/

      }
		else
		{
			goto CleanUp;
		}

   /* create the directory path and add to structure */

   strcpy(newClient->dirPath, "sys:clients\\");

                  
   /* find out the volume number for future calls */

   ccode = GetVolumeNumber(volName, &volNumber);
   if (ccode != 0)
   {   
   printf("\nError getting volume number");
   newClient->volumeNumber = 0;
   }
   else
   newClient->volumeNumber = volNumber;

   newClient->directoriesInUse = 
   newClient->filesInUse = 
   newClient->blocksInUse = 0L;

   /* Find the amount of disk space available on volume */

   ccode = GetAvailableUserDiskSpace("sys:", &availSpace);
   if (ccode != 0)
   {
   printf("\nError finding available disk space");

   }
      
   requestedSpaceIn4KBlocks = (newClient->spaceInUse*256);
   printf("\nRequested space in blocks: %d", requestedSpaceIn4KBlocks);

   if(requestedSpaceIn4KBlocks >= availSpace)
      newClient->spaceInUse = 0;

   if (newClient->spaceInUse)
      {
      strncat(newClient->dirPath, newClient->clientName, 9);	  

      if (mkdir(newClient->dirPath))
         {
         printf("\ncouldn't create client directory, may already exist");
         }


      /* Give rights to the newly created CLIENT directory */

      trusteeRightsMask = 0xFF;
      if ((ccode = AddTrustee(newClient->dirPath, newClient->objectID,
						      trusteeRightsMask)) != 0)
      printf("\nError granting client trustee rights");

      /* Add space restriction with amount of sectors */

      ccode = AddUserSpaceRestriction(-1, newClient->objectID, 
			  				      requestedSpaceIn4KBlocks);
      if (ccode !=0)
         {
         printf ("\nError adding disk space restriction");

         }
      }

   /* create dir. in the MAIL sub directory on server */

   if (!directoryClient)
      {

      sprintf(newClient->loginPath,"%s%08lx","sys:mail\\",
      newClient->objectID);


      if ((ccode = mkdir(newClient->loginPath)) !=0)
         sprintf(newClient->loginPath, "\0");
      else
         {

         /* Give rights to the newly created LOGIN directory */

         if ((ccode = AddTrustee(newClient->loginPath,newClient->objectID,
						         trusteeRightsMask)) != 0)
            printf("\nError granting login trustee rights");

         /* Create login script and copy it to Client's login dir. */

         sprintf(temp_str1,"%s%s",newClient->loginPath,"\\LOGIN");

         /* use temp string to keep file name separate from path */
         if ((fp = fopen(temp_str1,"w")) == NULL)
            printf("Can't open login file\n");
         else
            {
            fprintf(fp,"WRITE \"**********************************************\"\n");
            fprintf(fp,"WRITE \"* Welcome to BULLET's Disk Broker Service\"\n");

            sprintf(temp_str2,"%s%d%s\n\n","WRITE \"* You may use ",
							            newClient->spaceInUse,
							            " MB of disk space.\"");

            fprintf(fp,temp_str2);

            sprintf(map_str,"%s%s\n","MAP F:=SYS:CLIENTS\\",newClient->clientName);

            fprintf(fp,map_str);
            fprintf(fp,"MAP S1:=SYS:PUBLIC\n");
            fprintf(fp,"MAP s1:=SYS:LOGIN\n");
            fprintf(fp,"FIRE 3\n");
            fclose(fp);
            }
         }


      /* Add password property to new client object */

      ccode = ChangeBinderyObjectPassword(newClient->clientName, 
		  						      0x01,"\0",newClient->clientName);
      if (ccode)
      sprintf(newClient->loginPath, "\0");
      }

   if (directoryClient)
      {

      ccode = NWDS.InitBuf(context, DSV_MODIFY_ENTRY, userBuffer);
      if (ccode < 0)
         {
         printf("\nError initializing NWDS buffer (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }
      
      ccode = NWDS.PutChange(context, userBuffer,
                            DS_ADD_VALUE, "Login Script");
      if (ccode < 0)
         {
         printf("\nError putting change record (login script) in NWDS buffer \
               \n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      ccode = NWDS.PutAttrVal(context, userBuffer, SYN_STREAM, &loginStream);
      if (ccode < 0)
         {
         printf("\nError putting attribute value (login script) \
               in NWDS buffer\n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      ccode = NWDS.ModifyObject(context, newClient->clientName,
                               &iterationHandle, more, userBuffer);
      if (ccode < 0)
         {
         printf("\nError modifying user object (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }

      /* now we need to write login data to the users login stream */

      ccode = NWDS.OpenStream(context, newClient->clientName,
                             "Login Script", O_RDWR | O_CREAT, &handle);
      if (ccode < 0)
         {
         printf("\nError opening login stream (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }

      sprintf(temp_str2, "WRITE \"**********************************************\"\n");
      ccode = write(handle, temp_str2, strlen(temp_str2));
      sprintf(temp_str2,"WRITE \"* Welcome to BULLET's Disk Broker Service\"\n");
      ccode = write(handle, temp_str2, strlen(temp_str2));
      sprintf(temp_str2,"%s%d%s\n\n","WRITE \"* You may use ",
							            newClient->spaceInUse,
							            " MB of disk space.\"");
      ccode = write(handle, temp_str2, strlen(temp_str2));
      sprintf(map_str,"%s%s\n","MAP F:=SYS:CLIENTS\\",newClient->clientName);
      ccode = write(handle, map_str, strlen(map_str));
      sprintf(temp_str2,"MAP S1:=SYS:PUBLIC\n");
      ccode = write(handle, temp_str2, strlen(temp_str2));
      sprintf(temp_str2,"MAP s1:=SYS:LOGIN\n");
      ccode = write(handle, temp_str2, strlen(temp_str2));
      sprintf(temp_str2,"FIRE 3\n");
      ccode = write(handle, temp_str2, strlen(temp_str2));
      close(handle);
    

      ccode = NWDS.InitBuf(context, DSV_MODIFY_ENTRY, userBuffer);
      if (ccode < 0)
         {
         printf("\nError initializing NWDS buffer (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }


      ccode = NWDS.PutChange(context, userBuffer,
                            DS_ADD_ATTRIBUTE, "Home Directory");
      if (ccode < 0)
         {
         printf("\nError putting change record (home dir) in NWDS buffer \
               \n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      strcpy(homeDir, map_str);

      ccode = NWDS.PutAttrVal(context, userBuffer, SYN_CE_STRING, homeDir);
      if (ccode < 0)
         {
         printf("\nError putting attribute value (home dir) in NWDS buffer \
               \n\t(SetupClientInfo): %i", ccode);
         goto CleanUp;
         }

      ccode = NWDS.ModifyObject(context, newClient->clientName,
                               &iterationHandle, more, userBuffer);
      if (ccode < 0)
         {
         printf("\nError modifying user object (SetupClientInfo): %i",
                  ccode);
         goto CleanUp;
         }

      ccode = NWDS.GenerateObjectKeyPair(context,
                                        newClient->clientName,
                                        "", 0);
      if (ccode < 0)
         {
         printf("\nError getting public key (SetupClientInfo): %i", ccode);
         goto CleanUp;
         }
      }


   /* Init client status, shows client has all time left */	  

   newClient->shutdownStatus = 100;


CleanUp:
   switch(err)
      {
      case 2:
         NWDS.FreeBuf(userBuffer);
      default:
         break;
      }
//   if (ccode < 0)
     if (ccode)
      return(-1);

   return(0);	/* successful initialization of client object */
}

