
/**/
/*                                                                       */
/*       TLI Based Time Query Client using ConnectionLESS IPX            */
/*                                                                       */
/*   This routine will query the user for network and node address of    */
/*   any IPX time server.  It will then send a time request to           */
/*   that server using an IPX datagram.  If a response is received, the  */
/*   routine will set the local time to that received from the server    */
/*                                                                       */
/**/

#include <prolog.h>
#include <sys/types.h>
#include <ctype.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <nxt.h>
#include <niterror.h>
#include <fcntl.h>
#include <tiuser.h>
/* Now incorporated in tiuser.h #include <tispxipx.h> */
#include <sap.h>
#include <nwmisc.h>
#include "mrdnxt.h"   /* Missing nxt.h declarations */
#include "timeserv.h" /* Timeserver specific declarations */

/*   Routines found in ipxsubs.c   */

extern void huge receiverESR(void);
extern int  InitListenECB( WORD listenSocket, int num_of_ECB, WORD data_size );
extern int  ReceivedSPXPacket( ECB **receivedECB );
extern void RepostListenECB( ECB *listenECB, WORD listenSocket );
extern void DumpIPXpacket(IPXHeader *ipx_header, BYTE *data_buf, int dumpmax);
extern void GetIPXAddress( IPXAddress *addr );
extern int  strToHex(char *ascii, BYTE *hex, int length);
extern int  SendSPXPacket(WORD connectionID, void *data, WORD datalen);
extern int  EstablishConnection(IPXAddress *dest,WORD *connID,WORD mySocket);
extern int  TerminateConnection(WORD connectionID);


main() 
{
/*  */

	time_t		time_of_day;  /* returned from time()                       */
	time_t     *time_p;       /* used for conversion                        */
	struct tm	*time_block;  /* converted to local time block              */
	char        *time_zone;   /* ptr to TZ= string                          */
	IPXAddress  myaddress;    /* My network,node,socket                     */
	int 	      valid_reply;  /* A valid time reply received                */
	IPXAddress  server;       /* Server address                             */
	WORD        serverSocket; /* Temp WORD area for server socket           */
	int         received_data;  /* Boolean : did I receive data yet ?       */
	int         i;            /* scratch index                              */

/*   TLI structures and variables                                          */

	int               fd;              /* identifies our communication end  */
	struct t_bind     tbind;           /* to bind to a socket               */

	struct t_unitdata send_tunitdata;  /* sending datagram buffers, structs */
	IPX_OPTS          send_opts;       /* protocol options                  */
	int               send_flags;     
	unsigned char     ipx_data[512];   /* used for both send and receive    */

	struct t_unitdata recv_tunitdata;   /* receive datagram buffers, structs*/
	IPXAddress  recv_addr;              /* received from this address       */
	int               recv_flags;     

/*  */

/*   Get and display the old time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("\nThe time was %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (*time_zone == NULL)
		printf("Time zone was NULL\n");
	else
		printf("Time zone was %s\n\n",time_zone);

/*
 *   Ask the user which timeserver he wants to use. We do this because
 *   advertising/geting server only works from a workstation perspective
 *   if there is an intermediate router or if the workstation implements
 *   a complete SAP handler, which is not the case in this simple example.
 */
	printf("Please enter the timeserver address\n\n");
	GetIPXAddress(&server);
	serverSocket=TIME_SERVER_SOCKET;			 /* server socket is well known */
	memcpy(server.socket,&serverSocket,2);

/*
 *   Open a connection endpoint using the ipx protocol device.
 *   A unique id for this connection endpoint will be returned in fd.
 *   We open the endpoint as read and write, no info to be returned.
 *   We open no delay so we can abort from the keyboard.
 */
	if (( fd = t_open("/dev/nipx", O_RDWR|O_NDELAY, NULL )) == -1)
		{
		t_error("Open of IPX device failed, program aborted");
		return;
		}

/*
 *   Bind to a dynamic for socket chosen for this client to use.
 *   Our network and node need not be set, they are known to the provider
 *   and will be returned to us if bound successfully.  We use the
 *   same structure to pass and receive info from the provider.
 */
	memset(myaddress.socket,0,2);
	tbind.addr.len    = sizeof(IPXAddress);
	tbind.addr.maxlen = sizeof(IPXAddress);
	tbind.addr.buf    = (char *) &myaddress;
	tbind.qlen = 0;	/* qlen is meaningless for connectionless transports */

	if (t_bind(fd, &tbind, &tbind) == -1)
		{
		t_error("Bind to dynamic socket failed, program aborted");
		return;
		}

/*
 *   Display the local network and node returned from the t_bind and
 *   the well known server socket.
 */
	printf("TLI-IPX based time query client started on :\n\tNetwork number = 0x");
	for (i=0;i<4;i++)printf("%02x",myaddress.network[i]);
	printf("\n\tNode number    = 0x"); 
	for (i=0;i<6;i++)printf("%02x",myaddress.node[i]);
	printf("\n\tSocket number  = 0x");
	for (i=0;i<2;i++)printf("%02x",myaddress.socket[i]);
	printf("\n\nPress any key to abort program\n\n");

/*   Build and send a TIME REQUEST to the time server. */

/*   Setup the packet type in the options structure */

	send_opts.ipx_type        = IPX_PACKET_TYPE; 
	send_tunitdata.opt.buf    = (char *)&send_opts;
	send_tunitdata.opt.len    = sizeof(IPX_OPTS);
	send_tunitdata.opt.maxlen = sizeof(IPX_OPTS);

/*   Setup the address to send the request to */ 

	send_tunitdata.addr.buf    = (char *)&server;
	send_tunitdata.addr.len    = sizeof(IPXAddress);
	send_tunitdata.addr.maxlen = sizeof(IPXAddress);

/*   Setup the data buffer pointer and max length */

	send_tunitdata.udata.buf    = (char *)ipx_data;
	send_tunitdata.udata.maxlen = 512;
	ipx_data[0] = TIME_REQUEST;
	send_tunitdata.udata.len = 1;

	if (t_sndudata(fd, &send_tunitdata) == -1)
		{
		t_error("Fatal error, time request send unsuccessful");
		return;
		}

	printf("Time REQUEST sent, waiting for REPLY\n\n");

/*
 *   Listen on my socket for the REPLY, but only listen for
 *   a while.  Since IPX is datagram oriented and not guaranteed
 *   delivery, the REQUEST and/or the REPLY may never have made it.
 *   Since we opened in non-blocking mode we will look for data/sleep
 *   until some actually arrives. If a key is pressed, we abort.
 */
	valid_reply = FALSE;
	while (!valid_reply)
		{
		recv_tunitdata.addr.buf     = (char *)&recv_addr;
		recv_tunitdata.addr.maxlen  = sizeof(IPXAddress);
		recv_tunitdata.udata.buf    = (char *)ipx_data;
		recv_tunitdata.udata.maxlen = 512;
		recv_tunitdata.opt.maxlen   = 0;	  /* dont care to see them */

	/*
	 *   Loop, When something comes in t_rcvudata will return 0,
	 *   elsewise it should return -1 and t_errno will be set
	 *   to TNODATA ( no data available yet )
	 */
		received_data = FALSE;
		i=10;
		while ( ! received_data )
			{
			sleep(1); i--;
			if (i==0)
				{
				printf("Time out waiting for REPLY, program aborted\n\n");
				return;
				}
			if (kbhit())    
				{
				printf("Program canceled from keyboard\n");
				return;
				}
			if ( t_rcvudata(fd, &recv_tunitdata, &recv_flags ) == 0)
				received_data = TRUE;
			else
				{
				if (t_errno != TNODATA)
					{
					t_error("Fatal error during t_recvudata");
					return;
					}
				}
			}  /* end while ! received_data */

	/*
	 *   Check to see if this is really a time reply
	 *   if so, set the system time from that received in the packet.
	 */
		printf("Packet received on client socket from :\n\tNetwork number = 0x");
		for (i=0;i<4;i++)printf("%02x",recv_addr.network[i]);
		printf("\n\tNode number    = 0x"); 
		for (i=0;i<6;i++)printf("%02x",recv_addr.node[i]);
		printf("\n\n");

		if (ipx_data[0] != TIME_REPLY)
			printf("\nNon TIME REPLY received on socket, ignored\n\n");
		else
			{
			printf("\nTIME REPLY received on socket, setting system time\n\n");
			valid_reply = TRUE;

			time_zone=malloc(strlen(&ipx_data[1+sizeof(time_t)]) + 8 );
			strcpy(time_zone,"TZ=");
			strcat(time_zone,&ipx_data[1+sizeof(time_t)]);
			putenv(time_zone);      /* set TZ =               */
			tzset();                /* then make it permanent */
			free(time_zone);

			time_p      = (time_t *)&ipx_data[1];	 /* point to time in data */
			time_of_day = *time_p;                  /* get latest time       */
			stime(&time_of_day);                    /* and set it            */
			}

		} /* end while !valid_reply */

/*	   Get and display the new time   */

	time_of_day = time(NULL);
	time_block  = localtime(&time_of_day);	/* convert to a structure */
	printf("The time is now %s\n", asctime(time_block) );
	time_zone = getenv("TZ");
	if (time_zone == NULL)
		printf("Time zone is now NULL\n\n");
	else
		printf("Time zone is now %s\n\n",time_zone);

return;			
}  /* end main() */

